package com.vedant.mafia;

import android.content.Context;
import android.net.TrafficStats;

/**
 * Singleton class to monitor traffic generated by an app. More specifically, by a UID.
 * Generally every app has a single UID
 */
public class TrafficMonitor {
	private static TrafficMonitor instance;
	private final Context appContext;
	private long iTxBytes, iTxPackets, iRxBytes, iRxPackets;
	
	/**
	 * Starts recoding traffic stats when is method is first called. So to get traffic info per
	 * session (as long as app is in memory), call it from the "main" activity before any networking code,
	 * even if without any assignment.
	 * May throw unsupported exception if using the TrafficStats class is unsupported on this platform
	 * @param context: the context of calling application
	 */
	public static TrafficMonitor instance(Context context){
		if(instance == null)
			instance = new TrafficMonitor(context);
		return instance;
	}
	
	/**
	 * resets all the stored data.
	 */
	public void reset(){
		iTxBytes = TrafficStats.getUidTxBytes(getUid());
		iTxPackets = TrafficStats.getUidTxPackets(getUid());
		iRxBytes = TrafficStats.getUidRxBytes(getUid());
		iRxPackets = TrafficStats.getUidRxPackets(getUid());
	}
	
	/**
	 * methods to fetch data
	 */
	
	public long getPacketsTransferred(){
		return getPacketsRecieved() + getPacketsSent();
	}
	
	public long getBytesTransferred(){
		return getBytesSent() + getBytesRecieved();
	}
	
	public long getPacketsSent(){
		return TrafficStats.getUidTxPackets(getUid()) - iTxPackets;
	}
	
	public long getPacketsRecieved(){
		return TrafficStats.getUidRxPackets(getUid()) - iRxPackets;
	}
	
	public long getBytesRecieved(){
		return TrafficStats.getUidTxBytes(getUid()) - iTxBytes;
	}
	
	public long getBytesSent(){
		return TrafficStats.getUidTxBytes(getUid()) - iTxPackets;
	}
	
	private TrafficMonitor(Context context){
		appContext = context;
		reset();
		if(iTxBytes == TrafficStats.UNSUPPORTED)
			throw new UnsupportedOperationException("Use of TrafficStats class is unsupported in this platform");
	}

	private int getUid() {
		return appContext.getApplicationInfo().uid;
	}
}
